"use strict";
exports.__esModule = true;
var nodes_1 = require("decaffeinate-coffeescript2/lib/coffeescript/nodes");
var nodes_2 = require("../nodes");
var getLocation_1 = require("../util/getLocation");
var UnsupportedNodeError_1 = require("../util/UnsupportedNodeError");
var mapAny_1 = require("./mapAny");
var mapLiteral_1 = require("./mapLiteral");
function mapModuleDeclaration(context, node) {
    var _a = getLocation_1["default"](context, node), line = _a.line, column = _a.column, start = _a.start, end = _a.end, raw = _a.raw;
    if (node instanceof nodes_1.ImportDeclaration) {
        var clause = node.clause;
        if (!(clause instanceof nodes_1.ImportClause)) {
            throw new Error('Expected import clause as the clause for an import declaration.');
        }
        var defaultBinding = clause.defaultBinding && mapIdentifierSpecifier(context, clause.defaultBinding);
        var namespaceImport = null;
        var namedImports = null;
        if (clause.namedImports instanceof nodes_1.ImportNamespaceSpecifier) {
            namespaceImport = mapStarSpecifier(context, clause.namedImports);
        }
        else if (clause.namedImports instanceof nodes_1.ImportSpecifierList) {
            namedImports = clause.namedImports.specifiers.map(function (specifier) { return mapSpecifier(context, specifier); });
        }
        if (!node.source) {
            throw new Error('Expected non-null source for import.');
        }
        var source = mapSource(context, node.source);
        return new nodes_2.ImportDeclaration(line, column, start, end, raw, defaultBinding, namespaceImport, namedImports, source);
    }
    else if (node instanceof nodes_1.ExportNamedDeclaration) {
        if (node.clause instanceof nodes_1.ExportSpecifierList) {
            var namedExports = node.clause.specifiers.map(function (specifier) { return mapSpecifier(context, specifier); });
            var source = node.source ? mapSource(context, node.source) : null;
            return new nodes_2.ExportBindingsDeclaration(line, column, start, end, raw, namedExports, source);
        }
        else {
            var expression = mapAny_1["default"](context, node.clause);
            return new nodes_2.ExportNamedDeclaration(line, column, start, end, raw, expression);
        }
    }
    else if (node instanceof nodes_1.ExportDefaultDeclaration) {
        var expression = mapAny_1["default"](context, node.clause);
        return new nodes_2.ExportDefaultDeclaration(line, column, start, end, raw, expression);
    }
    else if (node instanceof nodes_1.ExportAllDeclaration) {
        if (!node.source) {
            throw new Error('Expected non-null source for star export.');
        }
        var source = mapSource(context, node.source);
        return new nodes_2.ExportAllDeclaration(line, column, start, end, raw, source);
    }
    else {
        throw new UnsupportedNodeError_1["default"](node);
    }
}
exports["default"] = mapModuleDeclaration;
function mapSource(context, coffeeSource) {
    var source = mapLiteral_1["default"](context, coffeeSource);
    if (!(source instanceof nodes_2.String)) {
        throw new Error('Expected string literal as import source.');
    }
    return source;
}
function mapIdentifierSpecifier(context, specifier) {
    if (specifier.alias) {
        throw new Error('Expected no alias for identifier specifier.');
    }
    return mapLiteralToIdentifier(context, specifier.original);
}
function mapStarSpecifier(context, specifier) {
    if (specifier.original.value !== '*') {
        throw new Error('Expected a star on the LHS of a star specifier.');
    }
    if (!specifier.alias) {
        throw new Error('Expected node on the RHS of star specifier.');
    }
    return mapLiteralToIdentifier(context, specifier.alias);
}
function mapSpecifier(context, specifier) {
    var _a = getLocation_1["default"](context, specifier), line = _a.line, column = _a.column, start = _a.start, end = _a.end, raw = _a.raw;
    var original = mapLiteralToIdentifier(context, specifier.original);
    var alias = specifier.alias ? mapLiteralToIdentifier(context, specifier.alias) : null;
    return new nodes_2.ModuleSpecifier(line, column, start, end, raw, original, alias);
}
function mapLiteralToIdentifier(context, literal) {
    var identifier = mapLiteral_1["default"](context, literal);
    if (!(identifier instanceof nodes_2.Identifier)) {
        throw new Error('Expected identifier in declaration.');
    }
    return identifier;
}
